
How to use: Turn on persistent "Table of Contents". This material is very dense.

Here’s a dedicated section focused on **keyboard behavior** and **form UX differences** between Android and iOS, which often affect layout stability, user frustration, and completion rates in mobile web apps:

## 🎹 Mobile Keyboard Behavior & Form UX Differences (Android vs iOS)

|Issue / Behavior|Android|iOS|UX Considerations & Fixes|
|---|---|---|---|
|**Keyboard Resize Effect**|Keyboard often resizes the viewport|Keyboard overlays the viewport, not shrinking it|Use `window.innerHeight` or `visualViewport.height` to detect height changes and adjust layout|
|**Fixed Elements with Keyboard**|Fixed-position elements (e.g., sticky footer buttons) usually stay put|Fixed elements can jump or disappear when keyboard opens|Use `position: absolute` + JS repositioning on iOS when input is focused|
|**Input Field Focus**|Scrolls input into view properly|Sometimes fails to scroll input into view, especially inside modals or if near bottom|Use `scrollIntoView()` on focus, with a delay for smoother animation on iOS|
|**Auto Zoom on Input Focus**|No zoom if font-size ≥ 16px|Zooms in if font-size < 16px|Set `input, textarea { font-size: 16px; }` to prevent iOS zoom|
|**Next/Previous Button on Keyboard**|Shows navigation arrows to move between fields|Often hidden, inconsistent depending on form layout|Group fields with `form` tag and logical tab order; don’t break fields into multiple containers|
|**Keyboard Type Customization**|Respects `input[type]` and `inputmode`|Mostly follows type, but `inputmode` support is spotty|Use `type="tel"` or `inputmode="numeric"` for numbers, and test on both platforms|
|**Keyboard Dismissal**|Often dismisses on outside tap or scroll|Sometimes keeps keyboard open until input blur|Add `onBlur` handlers and/or allow tap-to-dismiss zones|
|**Accessory Toolbar (Done, Next, etc.)**|Often shows toolbar with action buttons|iOS always shows top toolbar with “Done”/“Next”|Design with extra bottom padding to prevent UI elements from being covered|

---

### 📐 Best Practices

- **Avoid fixed footers with inputs** — especially on iOS — unless you manually reposition them on keyboard open.
    
- **Debounce scroll and resize events** when the keyboard is open to avoid layout thrashing.
    
- **Use `visualViewport` API** for the most reliable cross-platform way to detect keyboard and adjust UI:
    
    ```js
    window.visualViewport?.addEventListener('resize', () => {
      const keyboardHeight = window.innerHeight - window.visualViewport.height;
      // Adjust UI as needed
    });
    ```
    
- **Use `scroll-padding-bottom`** on containers with `overflow: auto` so input fields are not obscured:
    
    ```css
    form {
      scroll-padding-bottom: 120px; /* Reserve space for keyboard */
    }
    ```
    

---

## **Code snippets and strategies** for building keyboard-safe input fields and modals

### 🔧 1. **Adjust Layout on Keyboard Open (Cross-Platform)**

Use the `visualViewport` API (widely supported) to detect keyboard presence and shift elements accordingly:

```js
if (window.visualViewport) {
  window.visualViewport.addEventListener('resize', () => {
    const keyboardHeight = window.innerHeight - window.visualViewport.height;
    const formWrapper = document.querySelector('.form-wrapper');

    if (keyboardHeight > 100) {
      // Keyboard likely open
      formWrapper.style.marginBottom = `${keyboardHeight}px`;
    } else {
      formWrapper.style.marginBottom = '0px';
    }
  });
}
```

> 🔹 You can apply this to modals or form containers to prevent them from being obscured by the keyboard.

---

### 💬 2. **Scroll Input Field into View on Focus**

iOS sometimes fails to bring inputs fully into view — this workaround helps:

```js
document.querySelectorAll('input, textarea').forEach(el => {
  el.addEventListener('focus', () => {
    setTimeout(() => {
      el.scrollIntoView({ behavior: 'smooth', block: 'center' });
    }, 300); // Delay allows keyboard animation to finish
  });
});
```

---

### 📱 3. **Prevent iOS Zoom on Input**

iOS zooms in on inputs with small font sizes. To prevent that:

```css
input, textarea {
  font-size: 16px;
}
```

> 🔸 Optional: You can scale back down the UI afterward using `transform: scale()` if it affects spacing too much.

---

### 🧼 4. **Dismiss Keyboard on Tap Outside Input (iOS fix)**

Keyboard sometimes sticks around unless you blur the field manually:

```js
document.addEventListener('touchstart', function (e) {
  const active = document.activeElement;
  if (active && (active.tagName === 'INPUT' || active.tagName === 'TEXTAREA') && !e.target.closest('input, textarea')) {
    active.blur();
  }
});
```

---

### 🧩 5. **Keyboard-Safe Modal Example (HTML/CSS + JS)**

#### HTML:

```html
<div class="modal">
  <form class="modal-content form-wrapper">
    <input type="text" placeholder="Your Name" />
    <input type="email" placeholder="Email Address" />
    <button type="submit">Submit</button>
  </form>
</div>
```

#### CSS:

```css
.modal {
  position: fixed;
  bottom: 0;
  width: 100%;
  background: white;
  padding: 16px;
  transition: margin-bottom 0.2s ease;
}
```

The same `visualViewport` detection (see snippet #1) would allow `.modal` to move upward when the keyboard is open.

### 🔧 STACK: Plain JavaScript Utility: `keyboardSafeAdjust()`

```js
function keyboardSafeAdjust(selector = '.keyboard-safe') {
  const target = document.querySelector(selector);

  if (!target || !window.visualViewport) return;

  const adjust = () => {
    const keyboardHeight = window.innerHeight - window.visualViewport.height;
    if (keyboardHeight > 100) {
      target.style.marginBottom = `${keyboardHeight}px`;
    } else {
      target.style.marginBottom = '0px';
    }
  };

  window.visualViewport.addEventListener('resize', adjust);
  window.visualViewport.addEventListener('scroll', adjust);
}
```

 ✅ Usage:

```html
<div class="keyboard-safe">
  <!-- Your input fields or modal -->
</div>

<script>
  keyboardSafeAdjust('.keyboard-safe');
</script>
```

### ⚛️ STACK: React Hook: `useKeyboardSafePadding`

```jsx
import { useEffect, useState } from 'react';

export function useKeyboardSafePadding() {
  const [bottomPadding, setBottomPadding] = useState(0);

  useEffect(() => {
    const updatePadding = () => {
      if (window.visualViewport) {
        const keyboardHeight = window.innerHeight - window.visualViewport.height;
        setBottomPadding(keyboardHeight > 100 ? keyboardHeight : 0);
      }
    };

    window.visualViewport?.addEventListener('resize', updatePadding);
    window.visualViewport?.addEventListener('scroll', updatePadding);
    return () => {
      window.visualViewport?.removeEventListener('resize', updatePadding);
      window.visualViewport?.removeEventListener('scroll', updatePadding);
    };
  }, []);

  return bottomPadding;
}
```

 ✅ Usage in Component:

```jsx
import React from 'react';
import { useKeyboardSafePadding } from './useKeyboardSafePadding';

function FormModal() {
  const bottomPadding = useKeyboardSafePadding();

  return (
    <div
      className="fixed bottom-0 w-full bg-white p-4 transition-all duration-200"
      style={{ paddingBottom: bottomPadding }}
    >
      <input type="text" placeholder="Enter something" className="block mb-2 w-full" />
      <button className="w-full bg-blue-500 text-white p-2 rounded">Submit</button>
    </div>
  );
}
```

---
----

### ✅ STACK: CSS-Only Techniques for Keyboard-Safe Layouts

Here’s a **CSS-only strategy** to make input areas and modals more **keyboard-safe** — no JavaScript — using modern CSS features like `scroll-padding`, `env()` insets, and responsive layout techniques.

#### 🧩 1. **Use `scroll-padding-bottom` on scrollable containers**

This ensures that focused input fields are scrolled into view _above_ the virtual keyboard.

```css
.form-wrapper {
  overflow-y: auto;
  max-height: 100vh;
  scroll-padding-bottom: 120px; /* Reserve space for virtual keyboard */
}
```

> This works especially well for forms in modals or side panels that can scroll independently.

---

#### 📱 2. **Use `env(safe-area-inset-*)` for iOS Notch and Keyboard**

This helps on iPhones with notches or when the dynamic island affects layout.

```css
body {
  padding-bottom: env(safe-area-inset-bottom);
}
```

Or, for a fixed-position footer or modal:

```css
.fixed-footer {
  position: fixed;
  bottom: 0;
  left: 0;
  right: 0;
  padding: 1rem;
  padding-bottom: calc(1rem + env(safe-area-inset-bottom));
  background-color: white;
}
```

> Helps avoid the footer being too close to (or covered by) the iOS bottom edge or keyboard.

---

#### 📏 3. **Avoid 100vh directly — use `100dvh` instead**

On mobile Safari, `100vh` includes browser chrome (even when collapsed), which breaks layouts. Use the `dvh` unit:

```css
.fullscreen {
  height: 100dvh;
}
```

> `100dvh` adjusts dynamically as the viewport changes — including when the keyboard opens.

---

#### 🛠 4. **Combine with `position: sticky` for simple footers/buttons**

Use `position: sticky` on buttons inside a scrollable form:

```css
.button-wrapper {
  position: sticky;
  bottom: 0;
  background: white;
  padding: 1rem;
}
```

> This avoids fixed-position bugs on iOS and keeps the button visible when scrolling.

---

#### Example: Keyboard-Safe Modal Layout (CSS Only)

```html
<div class="form-wrapper">
  <form>
    <input type="text" placeholder="Your name" />
    <input type="email" placeholder="Email" />
    <div class="button-wrapper">
      <button type="submit">Submit</button>
    </div>
  </form>
</div>
```

```css
.form-wrapper {
  max-height: 100dvh;
  overflow-y: auto;
  scroll-padding-bottom: 120px;
}

.button-wrapper {
  position: sticky;
  bottom: 0;
  padding: 1rem;
  background: white;
}
```

---

### ✅ STACK: Tailwind CSS - Keyboard-Safe Modal Layout


Here’s a **Tailwind CSS version** of the keyboard-safe modal layout — optimized for mobile and PWA use. It uses utilities like `sticky`, `scroll-pb-[value]`, and `h-[100dvh]` for full support across Android and iOS.
#### 🧩 HTML Structure

```html
<div class="max-h-[100dvh] overflow-y-auto scroll-pb-32 px-4 py-6">
  <form class="space-y-4">
    <input type="text" placeholder="Your name" class="w-full border rounded p-2 text-base" />
    <input type="email" placeholder="Email" class="w-full border rounded p-2 text-base" />

    <div class="sticky bottom-0 bg-white pt-4 pb-[env(safe-area-inset-bottom)]">
      <button type="submit" class="w-full bg-blue-600 text-white rounded-lg py-3 text-lg shadow">
        Submit
      </button>
    </div>
  </form>
</div>
```

#### 🔍 Tailwind Highlights:

|Feature|Class Used|Purpose|
|---|---|---|
|`max-h-[100dvh]`|Uses dynamic viewport height to avoid 100vh bugs on iOS||
|`scroll-pb-32`|Adds scroll padding so keyboard doesn’t obscure inputs||
|`sticky bottom-0`|Keeps the button pinned at the bottom when scrolling||
|`pb-[env(safe-area-inset-bottom)]`|Adds notch-safe bottom padding for iOS||
|`shadow`, `rounded`, `py-3`|Polished, touch-friendly button styling||

---

#### 📱 Bonus: Optional Autofocus Scrolling Fix (JS-enhanced Tailwind)

If needed, add this JS snippet to ensure input scrolls into view on iOS:

```js
document.querySelectorAll('input, textarea').forEach(el => {
  el.addEventListener('focus', () => {
    setTimeout(() => el.scrollIntoView({ behavior: 'smooth', block: 'center' }), 300);
  });
});
```
